/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.sis.prediction;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.common.graph.INodeID;
import de.tudarmstadt.maki.simonstrator.api.component.ComponentNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSComponent;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSDataCallback;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSInfoProperties;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSInformationProvider.SiSProviderHandle;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSRequest;
import de.tudarmstadt.maki.simonstrator.api.component.sis.exception.InformationNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.sis.prediction.impl.NoRoadSegmentPrediction;
import de.tudarmstadt.maki.simonstrator.api.component.sis.type.PredictionContainer;
import de.tudarmstadt.maki.simonstrator.api.component.sis.type.PredictionSiSType;
import de.tudarmstadt.maki.simonstrator.api.component.sis.type.SiSType;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 03.01.2018
 *
 */
public class DefaultSiSPredictionManager implements SiSPredictionManager {

	private Map<SiSType<? extends Object>, Prediction<? extends Object>> availablePredictions = new HashMap<>();

	private Host host;

	private SiSComponent _siSComponent;

	public DefaultSiSPredictionManager(Host host) {
		this.host = host;
		
		Prediction<? extends Object> prediction = new NoRoadSegmentPrediction();
		availablePredictions.put(prediction.getBasicSiSType(), prediction);
	}

	@Override
	public void initialize() {
		try {
			_siSComponent = getHost().getComponent(SiSComponent.class);

			for (Entry<SiSType<? extends Object>, Prediction<? extends Object>> entry : availablePredictions
					.entrySet()) {
				PredictionSiSType<? extends Object> predictionSysType = entry.getKey().getPredictionSysType();
				if (predictionSysType != null) {
					_siSComponent.provide().nodeState(predictionSysType,
							new PredictionSiSDataCallback<>(predictionSysType));
				}
			}
		} catch (ComponentNotAvailableException e) {
		}
	}

	@Override
	public void shutdown() {

	}

	@Override
	public Host getHost() {
		return host;
	}

	@Override
	public <T> PredictionContainer<T> predict(INodeID pNodeID, PredictionSiSType<T> pType, T pValue, long pTime)
			throws InformationNotAvailableException {
		if (availablePredictions.containsKey(pType.getOriginalSiSType())) {
			Prediction prediction = availablePredictions.get(pType.getOriginalSiSType());
			return prediction.predict(pNodeID, pValue, pTime);
		}
		throw new InformationNotAvailableException();
	}

	@SuppressWarnings("rawtypes")
	private class PredictionSiSDataCallback<T extends Object> implements SiSDataCallback<PredictionContainer> {

		private PredictionSiSType<T> _sisType;

		public PredictionSiSDataCallback(PredictionSiSType<T> sisType) {
			_sisType = sisType;
		}

		@Override
		public PredictionContainer<T> getValue(INodeID pNodeID, SiSProviderHandle pProviderHandle)
				throws InformationNotAvailableException {

			T observation = _siSComponent.get().localObservationOf(pNodeID, _sisType.getOriginalSiSType(),
					SiSRequest.NONE);
			long timestamp = _siSComponent.get()
					.localObservationOfInfo(pNodeID, _sisType.getOriginalSiSType(), SiSRequest.NONE)
					.getLastUpdateTimestamp();

			return predict(pNodeID, _sisType, observation, timestamp);
		}

		@Override
		public Set<INodeID> getObservedNodes() {
			return _siSComponent.get().getObservedNodes(_sisType.getOriginalSiSType(), SiSRequest.NONE);
		}

		@Override
		public SiSInfoProperties getInfoProperties() {
			return new SiSInfoProperties();
		}
		
	}
	
}
