/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.vehicular.information;

import java.util.ArrayList;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.pubsub.Notification;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;

/**
 * Wrapper class for a notification.
 * 
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 14.11.2017
 *
 */
public class NotificationBasedVehicularPointInformation implements VehicularPointInformation<Object> {

	private Notification _notification;

	public NotificationBasedVehicularPointInformation(Notification pNotification) {
		_notification = pNotification;
	}

	@Override
	public Location getLocation() {
		return _notification.getAttribute(AvailableInformationAttributes.POSITION.getAttributeID(), Location.class)
				.getValue();
	}

	@Override
	public long getDetectionDate() {
		return _notification.getAttribute(AvailableInformationAttributes.DATE.getAttributeID(), Long.class).getValue();
	}

	@Override
	public <T> void setAttribute(AvailableInformationAttributes pKey, T pValue) {
		throw new UnsupportedOperationException(
				getClass().getSimpleName() + " is only a container class, no changes allowed!");
	}

	@Override
	public <T> T getAttribute(AvailableInformationAttributes pKey) {
		return (T) _notification.getAttribute(pKey.getAttributeID(), Object.class).getValue();
	}

	@Override
	public List<AvailableInformationAttributes> getAvailableAttributes() {
		return new ArrayList<>();
	}

	@Override
	public <T> boolean hasAttribute(AvailableInformationAttributes pKey) {
		return false;
	}

	@Override
	public Object getValue() {
		return _notification.getAttribute(AvailableInformationAttributes.VALUE.getAttributeID(), Object.class)
				.getValue();
	}

	@Override
	public RoadNetworkEdge getEdge() {
		return _notification.getAttribute(AvailableInformationAttributes.EDGE.getAttributeID(), RoadNetworkEdge.class)
				.getValue();
	}

	@Override
	public PointInformation<Object> clonePointInformation() {
		throw new AssertionError("Cannot be cloned");
	}

	@Override
	public void setDetectionTime(long _detectionTime) {
		throw new UnsupportedOperationException("Cannot set date");
	}

	@Override
	public PointInformation createDefaultInformation() {
		throw new UnsupportedOperationException("Cannot create default");
	}
}
