/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular.information;

import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;

/**
 * An information in a vehicular scenario is always associated with a location
 * and a creation time. Those properties are crucial to evaluate the existence
 * of an object.
 * 
 * @author Tobias Meuser
 * @version 1.0.0
 *
 */
public interface PointInformation<S extends Object> extends Cloneable {

	/**
	 * This method provides the location of the detected event.
	 * 
	 * @return The location of the event.
	 */
	Location getLocation();

	/**
	 * This method provides the detection date of the event. This is required to
	 * decide on the freshness of a detected event.
	 * 
	 * @return The detection timestamp of the event.
	 */
	long getDetectionDate();

	<T extends Object> void setAttribute(AvailableInformationAttributes pKey, T pValue);

	<T extends Object> T getAttribute(AvailableInformationAttributes pKey);

	List<AvailableInformationAttributes> getAvailableAttributes();

	<T extends Object> boolean hasAttribute(AvailableInformationAttributes pKey);

	S getValue();

	PointInformation<S> clonePointInformation();

	void setDetectionTime(long _detectionTime);

	PointInformation createDefaultInformation();
}
